<?php

namespace FiloBlu\CheckoutExtend\ConfigProvider;

use Magento\Checkout\Model\ConfigProviderInterface;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Store\Model\ScopeInterface;
use Magento\Store\Api\StoreResolverInterface;

class CheckoutConfigProvider implements ConfigProviderInterface
{
    const CONFIG_PATH_AUTO_COMPLETE_ENABLED   = 'filoblu_checkoutextend_section/auto_complete/enabled';
    const CONFIG_PATH_AUTO_COMPLETE_API_KEY   = 'filoblu_checkoutextend_section/auto_complete/api_key';

    /**
     * @var ScopeConfigInterface
     */
    private $scopeConfig;

      /**
     * @var array
     */
    private $regionOptions;

    /**
     * @var \Magento\Directory\Model\ResourceModel\Region\CollectionFactory
     */
    private $regionCollectionFactory;
    
        /**
     * @var StoreResolverInterface
     */
    private $storeResolver;
    
    /**
     * @param ScopeConfigInterface $scopeConfig
     */
    public function __construct(ScopeConfigInterface $scopeConfig,      \Magento\Directory\Model\ResourceModel\Region\CollectionFactory $regionCollection,
        StoreResolverInterface $storeResolver)
    {
        $this->scopeConfig = $scopeConfig;
                $this->regionCollectionFactory = $regionCollection;
        $this->storeResolver = $storeResolver;
    }

    /**
     * @return array
     */
    public function getConfig()
    {
        return [
            'addressAutoComplete' => [
                'enabled' => (bool)$this->scopeConfig->getValue(
                    self::CONFIG_PATH_AUTO_COMPLETE_ENABLED,
                    ScopeInterface::SCOPE_STORE
                ),
                'apiKey' => $this->scopeConfig->getValue(
                    self::CONFIG_PATH_AUTO_COMPLETE_API_KEY,
                    ScopeInterface::SCOPE_STORE
                ),
                'region_options' => $this->getRegionOptions()
            ]
        ];
    }
    
    private function getRegionOptions()
    {
        if (!isset($this->regionOptions)) {
            $this->regionOptions = $this->regionCollectionFactory->create()->addAllowedCountriesFilter(
                $this->storeResolver->getCurrentStoreId()
            );
        }

        return $this->regionToOptionArray($this->regionOptions);
    }
    
    private function regionToOptionArray($regionCollection)
    {
        $options = [];
        $propertyMap = [
            'value' => 'region_id',
            'code' => 'code',
            'country_id' => 'country_id',
        ];

        foreach ($regionCollection as $item) {
            $option = [];
            foreach ($propertyMap as $code => $field) {
                $option[$code] = $item->getData($field);
            }
//            $option['label'] = $item->getName();
            $options[] = $option;
        }

        return $options;
    }
}
